﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Net;
using System.Text;
using System.Windows.Forms;
using EosClocks;
using Microsoft.Win32;

namespace EOSClockDemo
{
    public partial class FrmActiveNetworkModules : Form
    {
        private bool _isFirstFormActivate = true;

        private class ActiveModulesItemComparer : IComparer
        {
            private int col;
            public ActiveModulesItemComparer()
            {
                col = 0;
            }
            public ActiveModulesItemComparer(int column)
            {
                col = column;
            }
            public int Compare(object x, object y)
            {
                return String.Compare(((ListViewItem)x).SubItems[col].Text, ((ListViewItem)y).SubItems[col].Text);
            }
        }


        public FrmActiveNetworkModules()
        {
            InitializeComponent();
            activeModules.Sorting = SortOrder.None;
            activeModules.ListViewItemSorter = new ActiveModulesItemComparer(2);
        }

        private void FrmActiveNetworkModules_Activated(object sender, EventArgs e)
        {
            if (_isFirstFormActivate)
            {
                accessMode.SelectedIndex = 0;
                _isFirstFormActivate = false;
            }
        }

        private void AddTibboDSToList(TibboDS ds, string ipAddress, int itemIndex)
        {
            string moduleinfo = "No status info available";
            ListViewItem li;
            if (itemIndex == -1)
                li = new ListViewItem();
            else
            {
                li = activeModules.Items[itemIndex];
                li.SubItems.Clear();
                li.ImageIndex = -1;
            }
            if (ds == null)
            {
                li.SubItems.Add(string.Empty);
                li.SubItems.Add(ipAddress);
                li.SubItems.Add(string.Empty);
                li.SubItems.Add("No response received from this IP-address");
            }
            else
            {
                li.SubItems.Add(ds.MACAddress);
                li.SubItems.Add(ds.IPAddress);
                li.SubItems.Add(ds.OwnerName + " / " + ds.DeviceName);
                li.ImageIndex = 0;
                if (ds.ErrorStatus != DSErrorStatus.NoErrorsDetected)
                {
                    li.ImageIndex = 2;
                    moduleinfo = "Error mode";
                }
                else if (ds.ProgrammingMode != DSProgrammingMode.None)
                {
                    li.ImageIndex = 4;
                    moduleinfo = "Firmware upload mode";
                }
                else if (ds.IPAddressStatus == DSIPAddressStatus.NotObtainedYet)
                {
                    li.ImageIndex = 3;
                    moduleinfo = "IP-address not obtained";
                }
                else
                {
                    li.ImageIndex = 1;
                    moduleinfo = "Normal state";
                }

                li.StateImageIndex = -1;
                switch (ds.DataConnectionStatus)
                {
                    case DSDataConnectionStatus.SendingARPOrEstablishingPPPLink:
                        li.StateImageIndex = 0;
                        moduleinfo += " - " + "sending ARP requests";
                        break;
                    case DSDataConnectionStatus.BeingEstablished:
                        li.StateImageIndex = 1;
                        moduleinfo += " - " + "Opening";
                        break;
                    case DSDataConnectionStatus.TCPConnectionEstablishedOrBeingClosed:
                    case DSDataConnectionStatus.UDPConnectionEstablished:
                        if ((ds.EthernetToSerialBufferOverflowState == DSOverflowState.OverflowDetected) ||
                            (ds.SerialToEthernetBufferOverflowState == DSOverflowState.OverflowDetected))
                        {
                            li.StateImageIndex = 3;
                            moduleinfo += " - " + "Established (or being closed), overrun detected";
                        }
                        else
                        {
                            li.StateImageIndex = 2;
                            moduleinfo += " - " + "Established (or being closed), no overrun";
                        }
                        break;
                    case DSDataConnectionStatus.ResetByRemoteHost:
                        li.StateImageIndex = 4;
                        moduleinfo += " - " + "Reset";
                        break;
                    default:
                        li.StateImageIndex = -1;
                        break;
                }
                li.SubItems.Add(moduleinfo);
            }
            if (itemIndex == -1)
                activeModules.Items.Add(li);
        }

        private void PreparePanleWait()
        {
            panleWait.Left = (activeModules.Width - panleWait.Width) / 2;
            panleWait.Top = activeModules.Top + ((activeModules.Height - panleWait.Height) / 2);
        }

        private void LoadActiveNetworkModules()
        {
            PreparePanleWait();
            panleWait.Visible = true;
            moduleStatusInfo.Text = string.Empty;
            activeModules.Items.Clear();
            Application.DoEvents();
            if (accessMode.SelectedIndex == 0)
            {
                foreach (TibboDS ds in TibboDSManager.ActiveDevices(IPAddress.Broadcast))
                {
                    AddTibboDSToList(ds, string.Empty, -1);
                    Application.DoEvents();
                }
            }
            else
            {
                // Load from Address Book
                //using (DALH::EOSDB eosDB = DALUtils.CreateNewEOSDB())
                //{
                //    foreach (DALH::TibboAddressBook ab in eosDB.TibboAddressBooks)
                //    {
                //        AddTibboDSToList(null, ab.TibboDSAddress.Trim(), -1);
                //    }
                //}
                Application.DoEvents();
                for (int i = 0; i < activeModules.Items.Count; i++)
                {
                    AddTibboDSToList(TibboDSManager.GetTibboDSState(
                        IPAddress.Parse(activeModules.Items[i].SubItems[2].Text)),
                        activeModules.Items[i].SubItems[2].Text, i);
                    Application.DoEvents();
                }
            }
            panleWait.Visible = false;
        }

        private void btnRefresh_Click(object sender, EventArgs e)
        {
            LoadActiveNetworkModules();
        }

        public string GetSelectedIPAddress()
        {
            return activeModules.SelectedItems[0].SubItems[2].Text;
        }

        private void btnOk_Click(object sender, EventArgs e)
        {
            if (activeModules.SelectedItems.Count == 0)
            {
                MessageBox.Show("Please Select One Item", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                DialogResult = DialogResult.None;
                return;
            }
            DialogResult = DialogResult.OK;
        }

        private void activeModules_DoubleClick(object sender, EventArgs e)
        {
            btnOk_Click(null, null);
        }

        private void activeModules_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (activeModules.SelectedItems.Count == 0)
                moduleStatusInfo.Text = string.Empty;
            else
                moduleStatusInfo.Text = activeModules.SelectedItems[0].SubItems[4].Text;
        }

        private void accessMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            btnAdd.Visible = accessMode.SelectedIndex != 0;
            btnEdit.Visible = btnAdd.Visible;
            btnDelete.Visible = btnAdd.Visible;
            btnImportDSManagerAddressBook.Visible = btnAdd.Visible;
            LoadActiveNetworkModules();
        }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            FrmIPAddress fmrIPAddress = new FrmIPAddress();
            if (fmrIPAddress.ShowDialog() == DialogResult.OK)
            {
                string ip = fmrIPAddress.IPAddress;

                //Save to Address Book
                //using (DALH::EOSDB eosDB = DALUtils.CreateNewEOSDB())
                //{
                //    DALH::TibboAddressBook ab = eosDB.GetTibboAddressBook(ip);
                //    if (ab == null)
                //    {
                //        ab = new DALH::TibboAddressBook();
                //        ab.TibboDSAddress = ip;
                //        eosDB.TibboAddressBooks.InsertOnSubmit(ab);
                //        eosDB.SubmitChanges();
                //    }
                //    else
                //    {
                //        MessageBox.Show(AppResources.InvalidAddBookEntryStr, AppResources.ErrorStr,
                //             MessageBoxButtons.OK, MessageBoxIcon.Error);
                //        return;
                //    }
                //}
                AddTibboDSToList(TibboDSManager.GetTibboDSState(IPAddress.Parse(ip)), ip, -1);
            }
        }

        private void btnEdit_Click(object sender, EventArgs e)
        {
            if (activeModules.SelectedItems.Count == 0)
                return;
            string oldIP = GetSelectedIPAddress();
            FrmIPAddress fmrIPAddress = new FrmIPAddress();
            fmrIPAddress.IPAddress = oldIP;
            if (fmrIPAddress.ShowDialog() == DialogResult.OK)
            {
                string ip = fmrIPAddress.IPAddress;
                if (ip == oldIP)
                    return;
                // Save To Address Book
                //using (DALH::EOSDB eosDB = DALUtils.CreateNewEOSDB())
                //{
                //    DALH::TibboAddressBook ab = eosDB.GetTibboAddressBook(oldIP);
                //    if (ab != null)
                //    {
                //        DALH::TibboAddressBook ab1 = eosDB.GetTibboAddressBook(ip);
                //        if (ab1 == null)
                //        {
                //            ab.TibboDSAddress = ip;
                //            eosDB.SubmitChanges();
                //        }
                //        else
                //        {
                //            MessageBox.Show(AppResources.InvalidAddBookEntryStr, AppResources.ErrorStr,
                //                 MessageBoxButtons.OK, MessageBoxIcon.Error);
                //            return;
                //        }
                //    }
                //}
                AddTibboDSToList(TibboDSManager.GetTibboDSState(IPAddress.Parse(ip)), ip,
                                 activeModules.SelectedItems[0].Index);
            }
        }

        private void btnDelete_Click(object sender, EventArgs e)
        {
            if (activeModules.SelectedItems.Count == 0)
                return;
            if (MessageBox.Show("Are you Sure,Delete Selected Item? ", "Confirmation",
                MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
                return;

            // Remove from Address Book
            //using (DALH::EOSDB eosDB = DALUtils.CreateNewEOSDB())
            //{
            //    DALH::TibboAddressBook ab = eosDB.GetTibboAddressBook(GetSelectedIPAddress());
            //    if (ab != null)
            //    {
            //        eosDB.TibboAddressBooks.DeleteOnSubmit(ab);
            //        eosDB.SubmitChanges();
            //    }
            //}
            activeModules.Items.RemoveAt(activeModules.SelectedItems[0].Index);
        }

        private void btnImportDSManagerAddressBook_Click(object sender, EventArgs e)
        {
            PreparePanleWait();
            panleWait.Visible = true;
            Application.DoEvents();
            RegistryKey reg1 = Registry.LocalMachine.OpenSubKey("SOFTWARE", false);
            RegistryKey reg2 = reg1.OpenSubKey("Tibbo", false);
            RegistryKey reg3 = reg2.OpenSubKey("DS Manager", false);
            string[] subKeyNames = reg3.GetSubKeyNames();
            List<string> ips = new List<string>();
            foreach (string sKey in subKeyNames)
            {
                RegistryKey reg4 = reg3.OpenSubKey(sKey, false);
                int value = (int)reg4.GetValue("DestinationIp");
                string hxValue = value.ToString("x").PadLeft(8, '0');
                string ip =
                    Convert.ToInt32(hxValue.Substring(0, 2), 16).ToString() + "." +
                    Convert.ToInt32(hxValue.Substring(2, 2), 16).ToString() + "." +
                    Convert.ToInt32(hxValue.Substring(4, 2), 16).ToString() + "." +
                    Convert.ToInt32(hxValue.Substring(6, 2), 16).ToString();
                ips.Add(ip);
                reg4.Close();
            }
            reg3.Close();
            reg2.Close();
            reg1.Close();
            // Save to Address Book
            //using (DALH::EOSDB eosDB = DALUtils.CreateNewEOSDB())
            //{
            //    for (int i = 0; i < ips.Count; i++)
            //    {
            //        DALH::TibboAddressBook ab = eosDB.GetTibboAddressBook(ips[i]);
            //        if (ab == null)
            //        {
            //            ab = new DALH::TibboAddressBook();
            //            ab.TibboDSAddress = ips[i];
            //            eosDB.TibboAddressBooks.InsertOnSubmit(ab);
            //            eosDB.SubmitChanges();
            //        }
            //        else
            //            ips[i] = string.Empty;
            //    }
            //}
            for (int i = 0; i < ips.Count; i++)
                if (ips[i] != string.Empty)
                    AddTibboDSToList(TibboDSManager.GetTibboDSState(
                        IPAddress.Parse(ips[i])), ips[i], -1);
            panleWait.Visible = false;
        }

    }


}
